#include "reg52.h"

typedef unsigned char u8;
typedef unsigned int u16;

sbit SDA=P1^7;
sbit SCL=P1^6;

#define  FOSC  11059200UL
#define  BRT  (65536 - FOSC / 115200 / 4)

sfr  AUXR  =  0x8e;
sfr  T2H  =  0xd6;
sfr  T2L  =  0xd7;

void delay(u16 i)
{
	while(i--);
	
}

void I2C_Start()
{
	SCL=1;
	SDA=1;
	delay(1);
	SDA=0;
	delay(1);
	SCL=0;
	delay(1);
}

void I2C_Stop()
{
	SDA=0;
	SCL=0;
	delay(1);
	SCL=1;
	delay(1);
	SDA=1;
	delay(1);
}

u8 I2C_Wait_ACK()
{
	u8 Err;
	SDA=1;
	delay(1);
	SCL=1;
	delay(1);
	while(SDA)
	{
		Err++;
		if(Err>200)
		{
			I2C_Stop();
			return 0;
		}
	}
	SCL=0;
	delay(1);
	return 1;
}

void I2C_ACK()
{
	 SCL=0;
	 SDA=0;
	 delay(1);
	 SCL=1;
	 delay(1);
	 SCL=0;
	 delay(1);
}

void I2C_NACK()
{
	SCL=0;
	SDA=1;
	delay(1);
	SCL=1;
	delay(1);
  SCL=0;
	delay(1);
}

void I2C_Send_Byte(u8 tx)
{
	u8 i;
	SCL=0;
	for(i=0;i<=7;i++)
	{
		 SDA=(tx&0x80)>>7;
		 tx<<=1;

		 delay(1);
		 SCL=1;
		 delay(1);
		 SCL=0;
	}
}

u8 I2C_Read_Byte(u8 ack)
{
	u8 a;
	u8 rec;
	SDA=1;
	for(a=0;a<=7;a++)
	{
		 SCL=0;
		 delay(1);
		 SCL=1;

		 rec<<=1;
		 rec|=SDA;
		
		 delay(1);
	}
	if(ack==1)
	   I2C_ACK();
	else 
	   I2C_NACK();
	return rec;
}

void VEML6070_Init()
{
	I2C_Start();
	I2C_Send_Byte(0x70);
	I2C_Wait_ACK();
	I2C_Send_Byte(0x06);
	I2C_Wait_ACK();
	I2C_Stop();
}

u16 VEML6070_Read()
{
	u8 H,L;
	I2C_Start();
	I2C_Send_Byte(0x73);
	I2C_Wait_ACK();
	H=I2C_Read_Byte(1);
	I2C_Stop();
	
	I2C_Start();
	I2C_Send_Byte(0x71);
	I2C_Wait_ACK();
	L=I2C_Read_Byte(1);
	I2C_Stop();
	
	return (u16)(H<<8)|L;
}

void UartInit()
{
	SCON = 0x50;
	T2L = BRT;
	T2H = BRT >> 8;
	AUXR = 0x15;
}


void SendInt(u16 dat)						 //??????
{
	u16 count[20];
	u8 i=0;
	while(dat)
	{
		count[i]=dat%10;
		dat=dat/10;
		i++;
	}
	while(i>0)
	{
		SBUF=count[i-1]+'0';
		while(!TI);
		TI=0;
		i--;
	}
	
}

void main()
{
	u16 UV;
	VEML6070_Init();
	UartInit();
	ES = 1;
	EA = 1;
	while(1)
	{
		I2C_Start(); //wake up
		I2C_Send_Byte(0x70);
		I2C_Wait_ACK();
		I2C_Send_Byte(0x06);
		I2C_Wait_ACK();
		I2C_Stop();
		
		delay(60000);
		delay(60000);
		delay(60000);
		delay(60000);
		UV=VEML6070_Read();
		SendInt(UV);
		SBUF='\n';		     //???
		while(!TI);
		TI=0;
		
		I2C_Start();  //sleep
		I2C_Send_Byte(0x70);
		I2C_Wait_ACK();
		I2C_Send_Byte(0x07);
		I2C_Wait_ACK();
		I2C_Stop();
		
		delay(60000);
		delay(60000);
		delay(60000);
		delay(60000);
		
			
	}
}